unit superreg2;

// Copyright  2002 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

// "Super" TRegistry class. The existing TRegistry will throw an
// exception if you try to read an Integer and the value is actually
// stored as a string, or try to write an Integer or String when
// the value is already present with a different type. TSuperReg
// can safely read and write String or Integer values, converting
// as necessary. It can also force-write a string or integer,
// by simply deleting any existing value first.

// The enhanced TSuperReg2 adds support for reading and writing
// binary data as strings of hex pairs.

interface
USES Windows, SysUtils, Registry;

type
  TSuperReg = class(TRegistry)
  public
    function ReadIntegerSafe(const Name: string; default : Integer)
      : Integer;
    function ReadStringSafe(const Name, default: string): string;
    procedure WriteIntegerSafe(const Name: string; Value: Integer);
    procedure WriteStringSafe(const Name, Value: string);
    procedure WriteIntegerForce(const Name: string; Value: Integer);
    procedure WriteStringForce(const Name, Value: string);
  end;

  TSuperReg2 = class(TSuperReg)
    function ByteArrayToString(Pb : PByteArray; Max : Integer) : String;
    function StringToByteArray(const S : String; Pb : PByteArray;
      Max : Integer) : Integer;
  public
    procedure WriteStringSafeNoBin(const Name, Value: string);
    function ReadBinString(const Name, default : String) : String;
    procedure WriteBinStringSafe(const Name, Value : String);
    procedure WriteBinStringForce(const Name, Value : String);
  end;

implementation

{ TSuperReg }

function TSuperReg.ReadIntegerSafe(const Name: string;
  default: Integer): Integer;
VAR
  rd  : TRegDataType;
  siz : Integer;
begin
  Result := Default;
  IF NOT ValueExists(Name) THEN Exit;
  Siz := GetData(Name, nil, 0, rd);
  CASE rd OF
    // String? Convert to Integer
    rdString,
    rdExpandString : Result := StrToIntDef(ReadString(Name), Default);
    // Integer? Just use it
    rdInteger      : Result := ReadInteger(Name);
    // Other? If it's 4 bytes in size, copy the data
    rdUnknown,
    rdBinary       : IF Siz = 4 THEN GetData(Name, @Result, 4, rd);
  END;
end;

function TSuperReg.ReadStringSafe(const Name, default: string): string;
VAR
  rd : TRegDataType;
  siz : Integer;
begin
  Result := Default;
  IF NOT ValueExists(Name) THEN Exit;
  Siz := GetData(Name, nil, 0, rd);
  CASE rd OF
    // String? Just read it
    rdString,
    rdExpandString : Result := ReadString(Name);
    // Integer? Convert to string
    rdInteger      : Result := IntToStr(ReadInteger(Name));
    // Other? Treat it as a string
    rdUnknown,
    rdBinary : begin
      SetLength(Result, Siz);
      GetData(Name, PChar(Result), Siz, rd);
      Result := StrPas(PChar(Result));
    end;
  END;
end;

procedure TSuperReg.WriteIntegerSafe(const Name: string;
  Value: Integer);
VAR
  DType : DWord;
  DSize : DWord;
begin
  IF RegQueryValueEx(CurrentKey, PChar(Name), nil, @DType, nil,
    @DSize) <> ERROR_SUCCESS THEN
    WriteInteger(Name, Value)
  ELSE
    begin
      CASE DType OF
        REG_SZ         : WriteString(Name, IntToStr(Value));
        REG_EXPAND_SZ  : WriteExpandString(Name, IntToStr(Value));
        REG_DWORD_BIG_ENDIAN :
                         WriteInteger(Name, MakeLong(Swap(HiWord(Value)),
                           Swap(LoWOrd(Value))));
        REG_DWORD      : WriteInteger(Name, Value);
        ELSE
          RegSetValueEx(CurrentKey, PChar(Name), 0, DType,
            @Value, 4);
      END;
    end;
end;

procedure TSuperReg.WriteStringSafe(const Name, Value: string);
VAR
  DType : DWord;
  DSize : DWord;
  I     : Integer;
begin
  IF RegQueryValueEx(CurrentKey, PChar(Name), nil, @DType, nil,
    @DSize) <> ERROR_SUCCESS THEN
    WriteString(Name, Value)
  ELSE
    begin
      I := StrToIntDef(Value, 0);
      CASE DType OF
        REG_SZ         : WriteString(Name, Value);
        REG_EXPAND_SZ  : WriteExpandString(Name, Value);
        REG_DWORD_BIG_ENDIAN :
                         WriteInteger(Name, MakeLong(Swap(HiWord(I)),
                           Swap(LoWOrd(I))));
        REG_DWORD      : WriteInteger(Name, I);
        ELSE
          RegSetValueEx(CurrentKey, PChar(Name), 0, DType,
            PChar(Value), Length(Value)+1);
      END;
    end;
end;

procedure TSuperReg.WriteIntegerForce(const Name: string;
  Value: Integer);
begin
  DeleteValue(Name);
  WriteInteger(Name, Value);
end;

procedure TSuperReg.WriteStringForce(const Name, Value: string);
begin
  DeleteValue(Name);
  WriteString(Name, Value);
end;

{ TSuperReg2 }

function TSuperReg2.ByteArrayToString(Pb: PByteArray; Max : Integer): String;
// Take the area of memory pointed to by Pb and display it as a
// series of hex bytes, separated by spaces
VAR N : Integer;
begin
  Result := '';
  FOR N := 0 TO Max-1 DO
    Result := Result + IntToHex(Pb^[N],2) + ' ';
  Result := Trim(Result);
end;

function TSuperReg2.StringToByteArray(const S: String; Pb: PByteArray;
  Max: Integer): Integer;
// Take a string of bytes represented as hex byte strings separated
// by spaces and put the corresponding bytes into the memory area
// pointed to by the argument PB, for no more than Max bytes.
const
  HexDigs : ARRAY[0..15] OF Char = '0123456789ABCDEF';
VAR
  N,P : Integer;
  B : Byte;
begin
  Result := 0;
  IF trim(S) = '' THEN Exit;
  N      := 0;
  B      := 0;
  REPEAT
    Inc(N);
    // If the current character is a hex digit, tack it onto the byte
    // we are building. Specifically, shift the existing byte's
    // digits left by 4 (multiply by 16) and add the new digit.
    P := Pos(upcase(S[N]), hexDigs)-1;
    IF P >= 0 THEN
      B := (B SHL 4) + P
    ELSE
      begin
        // If this is a non-digit, record the built-up byte and
        // reset it to 0.
        Pb^[Result] := B;
        B := 0;
        Inc(Result);
        IF Result = Max THEN
          Raise(Exception.Create('StringToByteArray: More bytes '+
            'than specified maximum.'));
        // Skip all non-digits (should be just one space)
        WHILE (N+1 < Length(S)) AND (Pos(upcase(S[N+1]), hexdigs) = 0) DO
          Inc(N);
      end;
  UNTIL N >= Length(S);
  // Record the last built-up byte
  Pb^[Result] := B;
end;

function TSuperReg2.ReadBinString(const Name, default: String): String;
VAR
  rd  : TRegDataType;
  siz : Integer;
  S   : String;
  I   : Integer;
  pb  : PByteArray;
begin
  Result := Default;
  IF NOT ValueExists(Name) THEN Exit;
  Siz := GetData(Name, nil, 0, rd);
  CASE rd OF
    // String? report as binary
    rdString,
    rdExpandString : begin
      S := ReadString(Name);
      Result := ByteArrayToString(PByteArray(PChar(S)), Length(S));
    end;
    // Integer? report as binary
    rdInteger      : begin
      I := ReadInteger(Name);
      Result := ByteArrayToString(PByteArray(@I), SizeOf(I));
    end;
    // Other? If it's 4 bytes in size, copy the data
    rdUnknown,
    rdBinary       : begin
      GetMem(pb, Siz);
      try
        ReadBinaryData(Name, pb^, Siz);
        Result := ByteArrayToString(pb, siz);
      finally
        FreeMem(pb);
      end;
    end;
  END;
end;

procedure TSuperReg2.WriteBinStringForce(const Name, Value: String);
VAR
  pb : PByteArray;
  siz : Integer;
begin
  DeleteValue(Name);
  Siz := Succ(Length(Value)) DIV 3;
  GetMem(pb, Siz);
  try
    StringToByteArray(Value, pb, Siz);
    WriteBinaryData(Name, pb^, Siz);
  finally
    FreeMem(pb);
  end;
end;

procedure TSuperReg2.WriteBinStringSafe(const Name, Value: String);
VAR
  DType : DWord;
  DSize : DWord;
  Siz   : Integer;
  pb    : PByteArray;
  IVal  : Integer;
begin
  Siz := Succ(Length(Value)) MOD 3;
  GetMem(pb, Siz);
  try
    StringToByteArray(Value, pb, Siz);
    IF Siz = 4 THEN move(pb^,iVal,4);
    IF RegQueryValueEx(CurrentKey, PChar(Name), nil, @DType, nil,
      @DSize) <> ERROR_SUCCESS THEN
      WriteBinaryData(Name, pb^, Siz)
    ELSE
      begin
        CASE DType OF
          REG_SZ         : WriteString(Name, PChar(pb));
          REG_EXPAND_SZ  : WriteExpandString(Name, PChar(pb));
          REG_DWORD_BIG_ENDIAN : begin
            IF siz <> 4 THEN
              Raise(Exception.Create('WriteBinStringSafe attempting to put '+
                IntToStr(siz)+ ' bytes in a 4 byte Big Endian Integer'));
             WriteInteger(Name, MakeLong(Swap(HiWord(iVal)),
               Swap(LoWOrd(iVal))));
          end;
          REG_DWORD      : begin
            IF siz <> 4 THEN
              Raise(Exception.Create('WriteBinStringSafe attempting to put '+
                IntToStr(siz)+ ' bytes in a 4 byte Integer'));
            WriteInteger(Name, iVal);
          end;
          ELSE
            RegSetValueEx(CurrentKey, PChar(Name), 0, DType,
              pb, siz);
        END;
      end;
  finally
    FreeMem(pb);
  end;
end;

procedure TSuperReg2.WriteStringSafeNoBin(const Name, Value: string);
VAR
  DType : DWord;
  DSize : DWord;
  I     : Integer;
begin
  IF RegQueryValueEx(CurrentKey, PChar(Name), nil, @DType, nil,
    @DSize) <> ERROR_SUCCESS THEN
    WriteString(Name, Value)
  ELSE IF DType = REG_BINARY THEN
    WriteStringForce(Name, Value)
  ELSE WriteStringSafe(Name, Value);
end;

end.


VAR
  DType : DWord;
  DSize : DWord;
  I     : Integer;
begin
  IF RegQueryValueEx(CurrentKey, PChar(Name), nil, @DType, nil,
    @DSize) <> ERROR_SUCCESS THEN
    WriteString(Name, Value)

